st7/
;----------------------------------------------------------
;Program demonstracyjny demonstrujacy uzycie enkodera 
; magnetycznego AS5040
;----------------------------------------------------------
;Enkoder dolaczony do:
;* PB0 - CSN (Chip Select)
;* PB1 - CLK (linia zegara SSI)
;* PB5 - DO (wyjscie danych SSI)
;----------------------------------------------------------

	#include "st7flite29.inc"

;segmenty pamieci
	BYTES
	segment byte at 80-FF 'ram0'
	segment byte at 100-1FF 'stack'
	segment byte at 200-27F 'ram1'
	segment byte at 1000-10FF 'eeprom'
	segment byte at E000-FFDF 'program'
	segment byte at FFE0-FFFF 'intvect'

	BYTES
;deklaracje zmiennych
	segment 'ram0'
.del	DS.B	1
.copya	DS.B	1
.posen	DS.B	2		;pozycja enkodera (dziesietnie)
.stateen	DS.B	1		;status enkodera
sub_a	DS.W
sub_b	DS.W
roznicaw	DS.W
decims	DS.B	6

				;odebrane bity statusu
	#define	OCF	stateen,#0
	#define	COF	stateen,#1
	#define	LIN	stateen,#2
	#define	MAGINC	stateen,#3
	#define	MAGDEC	stateen,#4
	#define	PARITY	stateen,#5

	#define	CSN	PBDR,#0	;linie sterujace enkoderem
	#define	CLK	PBDR,#1
	#define	DO	PBDR,#5


;deklaracje stalych
	WORDS
	segment 'program'
.txt1	DC.B	"ST7FLITE-29",$0
.txt2	DC.B	"test LCD",$0

;-------------------------------------------------------
; poczatek programu glownego, funkcja inicjalizacji
; stosu, portow I/O oraz zmiennych
;-------------------------------------------------------
.init
	ld	A,#$FF		;port A jako wyjsciowy
	ld	PADDR,A		;i stan niski na wszystkich wyjsciach
	ld	PAOR,A
	clr	A
	ld	PADR,A

	ld	A,#$03		;port B w calosci jako wejsciowy oprcz bitw CLK i CSN (bity 0, 1)
	ld	PBDDR,A
	ld	A,#$03		;zalaczenie rezystorow zasilajacych (bit 0, 1)
	ld	PBOR,A
	ld	PBDR,A
	
	ld	A,#$01		;f_CPU = f_OSC/32 @ 1MHz
	ld	MCCSR,A
	
	ld	A,#$12		;f_TIMER = f_CPU
	ld	ATCSR,A
        
	ld	A,#$DF		;konfiguracja 12-bitowego timera
	ld	ATRL,A		;autoreload, przerwanie co ok.1 ms
	ld	A,#$0F
	ld	ATRH,A
	ret
	

;-------------------------------------------------------
; funkcja obslugi przerwania timera B uzywana przez
; delay do odmierzania czasu
;-------------------------------------------------------
.timerb
	ld	A,ATCSR		;odczyt ATCSR w celu zerowania flagi przerwania
	ld	A,del		;do A wartosc opoznienia
	and	A,#$FF		;sprawdzenie, czy A=0
	jreq	timerb_exit
	dec	A
	ld	del,A
timerb_exit        
	iret


;-------------------------------------------------------
; funkcja realizujaca opoznienie "del" x 1ms (maks.255ms)
; wielkosc opoznienia w komorce "del"
;-------------------------------------------------------
.delay
	push	A		;zapamietanie akumlatora na stosie
	rim			;zalaczenie przerwan
d_loop
	ld	A,del		;zmienna "del" zmniejszana przez funkcje
	and	A,#$FF		;obslugi przerwania timer'a
	jreq	d_end		;w tej petli funkcja oczekuje na wartosc del=0
	jra	d_loop
d_end
	sim			;wylaczenie przerwan
	pop	A		;odtworzenie zawartosci akumulatora
	ret
	

;-------------------------------------------------------
; wyslanie danej do wyswietlacza, rejestr docelowy
; okreslany przez inne funkcje-tu tylko przeslanie
; zmienna wejsciowa w akumulatorze!
; RS=PA4,EN=PA7,PA0=DB4,PA1=DB5,PA2=DB6,PA3=DB7
;-------------------------------------------------------
.lcd_write
	ld	Y,A		;zapamietanie akumulatora
	ld	A,PADR		;odczyt stanu portu A (bo moga byc podlaczone do
				;wyprowadzen portow inne obwody,niz wyswietlacz)
	or	A,#$80		;ENABLE=1
	ld	PADR,A
	or	A,#$0F		;ustawienie bitw PA.0-4 na "1" dla operacji AND
	ld	PADR,A
	push	A		;zapamietanie akumulatora (tzn stanu PADR)
	ld	A,Y		;odtworzenie danej do przeslania
	swap	A		;zamiana polowek bajtu - starsza jako 1-sza
	or	A,#$F0		;ustawienie "nieznaczacej" polowy bajtu na "1"
				;dla operacji AND
	ld	copya,A		;zapamietanie zmiennej do przeslania
	pop	A		;odtworzenie stanu PADR
	and	A,copya		;iloczyn logiczny dla bezkolizyjnego zapisu danych
	ld	PADR,A		;wyprowadzenie polowki bajtu
	and	A,#$7F		;zmiana stanu ENABLE - zapis bajtu do LCD
	ld	PADR,A
	or	A,#$80		;ENABLE=1
	ld	PADR,A
	or	A,#$0F		;ustawienie bitw PA.0-4 na "1" dla operacji AND
	ld	PADR,A
	push	A		;zapamietanie stanu PADR
	ld	A,Y		;odtworzenie danej do przeslania
	or	A,#$F0		;ustawienie "nieznaczacej" polowy bajtu na "1"
				;dla operacji AND
	ld	copya,A		;zapamietanie zmiennej do przeslania
	pop	A		;odtworzenie stanu PADR
	and	A,copya		;iloczyn logiczny dla bezkolizyjnego zapisu danych
	ld	PADR,A		;wyprowadzenie polowki bajtu
	and	A,#$7F		;zmiana stanu ENABLE - zapis bajtu do LCD
	ld	PADR,A
	ld	A,#10		;pauza - LCD akceptuje dane
	ld	del,A
	callr	delay
	ret


;-------------------------------------------------------
; wyslanie zawartosci akumulatora do rejestru
; konfiguracji wyswietlacza LCD
;-------------------------------------------------------
.lcd_reg_write
	push	A		;zapamietanie zawartosci akumlatora
	ld	A,PADR
	and	A,#$6F		;RS(D4)=0,ENABLE(D7)=0
	ld	PADR,A
	pop	A		;wyslanie zawartosci akumulatora do lcd
	callr	lcd_write
	ret

;-------------------------------------------------------
; wyslanie zawartosci akumulatora do rejestru
; danych wyswietlacza LCD
;-------------------------------------------------------
.lcd_data_write
	push	Y
	push	A
	ld	A,PADR
	and	A,#$6F		;RS(D4)=0,ENABLE(D7)=0
	or	A,#$10		;RS(D4)=1
	ld	PADR,A
	pop	A
	callr	lcd_write
	pop	Y
	ret
	
;-------------------------------------------------------
; inicjalizacja wyswietlacza LCD w trybie interfejsu
; o dlugosci slowa 4 bity
;-------------------------------------------------------
.lcd_init
	ld	A,#100		;pauza okolo 100 ms
	ld	del,A
	callr	delay
	ld	A,PADR		;RS(D4)=0,ENABLE(D7)=0,dane=0
	and	A,#$60
	ld	PADR,A
	ld	Y,#3		;3-krotne przeslanie 0x03 do lcd
	or	A,#3
	ld	PADR,A
lcd_init_loop    
	ld	A,PADR		;odczyt stanu rejestru danych portu A
	or	A,#$80		;ENABLE=1
	ld	PADR,A
	nop
	nop
	and	A,#$7F		;ENABLE=0
	ld	PADR,A
	ld	A,#5		;opoznienie 5 milisekund
	ld	del,A
	call	delay
	dec	Y		;czy juz powtorzono 3x?
	jrne	lcd_init_loop
	ld	A,#2		;zapisanie "2" do rejestru kontrolnego
	callr	lcd_reg_write
	ld	A,#$28		;kolejne wartosci inicjujace ($28,8,1,6,$C)
	callr	lcd_reg_write
	ld	A,#8
	callr	lcd_reg_write	
	ld	A,#1
	callr	lcd_reg_write	
	ld	A,#6
	callr	lcd_reg_write	
	ld	A,#$C
	callr	lcd_reg_write	
	ret   

;-------------------------------------------------------
; kasowanie ekranu wyswietlacza LCD
;-------------------------------------------------------
.lcd_cls
	push	A
	ld	A,#1
	callr	lcd_reg_write
	pop	A
	ret


;-------------------------------------------------------
; ustawienie kursora na wspolrzednych X,Y               
; X i Y podawane w A w formacie A:Y<<4+X
;-------------------------------------------------------
.gotoaxy
	ld	Y,A		;zapamietanie akumulatora w rejestrze "Y"
	and	A,#$0F		;wyodrebnienie parametru X
	push	A		;zapamietanie parametru X na stosie
	ld	A,Y		;odtworzenie akumulatora, wyodrebnienie par.Y
	sra	A		;arytmetyczne przesuniecie w prawo
	sra	A		;najstarszy bit jest zastepowany przez "0"
	sra	A
	sra	A
	cp	A,#0		;skok wykonywany,jesli A=0 (dla Y=0)
	jreq	gzero
	cp	A,#1 		;jesli A=1
	jreq	gone
	cp	A,#2		;jesli A=2
	jreq	gtwo
	cp	A,#3		;jesli A=3
	jreq	gthree
	ret
;jesli Y=0
gzero
	ld	A,#$80
	ld	copya,A		;obliczenie offsetu adresu dla Y=0
	jra	gotoaxy_exit
gone                            
	ld	A,#$C0
	ld	copya,A		;dla Y=1
	jra	gotoaxy_exit
gtwo                     
	ld	A,#$94
	ld	copya,A		;dla Y=2
	jra	gotoaxy_exit
gthree                  
	ld	A,#$D4
	ld	copya,A		;dla Y=3
gotoaxy_exit
	pop	A
	add	A,copya
	call	lcd_reg_write
	ret


;-------------------------------------------------------
;odczyt danych z enkodera, pozycja zapisywana w zmiennej 
;<positen>, stan enkodera w zmiennej <stateen>
;-------------------------------------------------------
.read_as5040
	ld	A,PBDR
	or	A,#$03		;CSN(PB0)=1, CLK(PB1)=1
	ld	PBDR,A
	clr	posen		;zerowanie zmiennych statusu i pozycji
	clr	{posen+1}
	clr	stateen
	clr	A		;ustawienie kursora na 0,0
	call	gotoaxy
	bres	CSN		;zalaczenie wyboru ukladu
;odczyt 10 bitw pozycji
	ld	Y,#10		;odbierane bedzie 16 bitow, tu licznik
	bres	CLK		;zerowanie linii zegara
	nop
.read_as_loop
	bset	CLK		;ustawienie linii zegara, odczyt danych
	nop
	rcf			;zerowanie flagi CY
	bres	CLK		;zerowanie linii zegara
	btjf	DO,read_as_0	;odczytano "0", nie ustawiamy bitu - tylko przesuniecie
	scf			;odczytano "1", ustawiamy bit C i przesuwamy w lewo zmienna
	ld	A,#'1'
	jra	read_shift_1
.read_as_0
	ld	A,#'0'
.read_shift_1
	call	lcd_data_write
	rlc	posen		;przesuniecie w lewo o 1 bit (2 bajtow zmiennej posen), flaga
				;C wprowadzana na najmlodszej pozycji bitu
	rlc	{posen+1}	;teraz uwzgledniany stan bitu z mlodszego bajtu
	dec	Y		;sprawdzenie, czy odebrano 10 bitow
	jrne	read_as_loop

;odczyt 5 bitw statusu
	ld	A,#$10		;ustawienie kursora na 0,1
	call	gotoaxy
	ld	Y,#5		;metoda taka sama, jak wyzej, dla odczytu bitow danych
.read_as_status
	bset	CLK
	nop
	rcf
	bres	CLK
	btjf	DO,read_st_0
	scf
	ld	A,#'1'
	jra	read_shift_2
.read_st_0
	ld	A,#'0'
.read_shift_2
	call	lcd_data_write
	rlc	stateen
	dec	Y
	jrne	read_as_status

;16-ty bit (dla zgodnosci z PDF-em)
	bset	CLK
	nop
	rcf
	btjf	DO,read_st_0_1
	scf
	ld	A,#'1'
	jra	read_shift_2_1
.read_st_0_1
	ld	A,#'0'
.read_shift_2_1
	call	lcd_data_write
	rlc	stateen

;	bset	CLK		;wylaczenie enkodera
;	nop
	bset	CSN
	ret
	

;-------------------------------------------------------
; odejmowanie liczb 2-bajtowych
;-------------------------------------------------------
subw
	push	A		;zapamietanie rejestrw roboczych
	push	X
	ld	A,{sub_a+1} 	;pobranie mlodszego liczby a
	sub 	A,{sub_b+1} 	;dodanie mlodszego liczby b
	ld 	{roznicaw+1},A 	;zapamietanie mlodszego bajtu roznicy
	ld 	A,sub_a 		;pobranie starszego bajtu liczby a
	sbc 	A,sub_b 		;odjecie przeniesienia i starszego bajtu liczby b
	ld 	roznicaw,A 	;zapamietanie starszego bajtu roznicy
	pop 	X 		;odtworzenie rejestrw
	pop 	A
	ret


;-------------------------------------------------------
;konwersja na liczbe dziesietna; poszczegolne wagi
;umieszczone w komorkach tablicy DECIMS
;uwaga: funkcja nieco "na wyrost"! wynik pomiaru nigdy
;       nie przekroczy wartosci 1024 dec. (3FFh)
;-------------------------------------------------------
bin_2_dec
	ld	Y,#6		;zerowanie tablicy DECIMS
	clr	A
	clr	X
bin_2_dec_1
	ld	(decims,X),A
	inc	X
	dec	Y
	jrne	bin_2_dec_1
				;zaladowanie wyniku pomiaru do
	ld	A,{posen+1}	;komorek SUB_A
	ld	sub_a,A
	ld	A,posen
	ld	{sub_a+1},A
				;liczenie ile razy 10000 miesci sie w liczbie
	ld	A,#$27
	ld	sub_b,A
	ld	A,#$10
	ld	{sub_b+1},A
bin_2_10000
	callr	subw		;odejmowanie SUB_A-SUB_B
	jrc	bin_2_10000_end	;jesli C, to liczba<10000
	inc	decims		;zwiekszenie licznika 10000
	ld	A,roznicaw	;przeniesienie wynku do SUB_A
	ld	sub_a,A
	ld	A,{roznicaw+1}
	ld	{sub_a+1},A
	jra	bin_2_10000	;powtorne odejmowanie 10000

bin_2_10000_end			;liczenie ile razy 1000 miesci sie w liczbie
	ld	A,#3
	ld	sub_b,A
	ld	A,#$E8
	ld	{sub_b+1},A
bin_2_1000
	callr	subw
	jrc	bin_2_1000_end	;jesli C, to liczba<1000
	inc	{decims+1}	;zwiekszenie licznika 1000
	ld	A,roznicaw	;przeniesienie wynku do SUB_A
	ld	sub_a,A
	ld	A,{roznicaw+1}
	ld	{sub_a+1},A
	jra	bin_2_1000	;powtorne odejmowanie 1000

bin_2_1000_end			;liczenie ile razy 100 miesci sie w roznicy
	clr	sub_b
	ld	A,#100
	ld	{sub_b+1},A
bin_2_100
	callr	subw
	jrc	bin_2_100_end	;jesli C, to liczba<100
	inc	{decims+2}	;zwiekszenie licznika 100
	ld	A,roznicaw	;przeniesienie wynku do SUB_A
	ld	sub_a,A
	ld	A,{roznicaw+1}
	ld	{sub_a+1},A
	jra	bin_2_100	;powtorne odejmowanie 100

bin_2_100_end			;liczenie ile razy 10 miesci sie w roznicy
	clr	sub_b
	ld	A,#10
	ld	{sub_b+1},A
bin_2_10
	callr	subw
	jrc	bin_2_10_end	;jesli C, to liczba<10
	inc	{decims+3}	;zwiekszenie licznika 10
	ld	A,roznicaw	;przeniesienie wynku do SUB_A
	ld	sub_a,A
	ld	A,{roznicaw+1}
	ld	{sub_a+1},A
	jra	bin_2_10		;powtorne odejmowanie 10

bin_2_10_end
	ld	A,{sub_a+1}	;pozostalosc po odejmowaniu to jednostki
	ld	{decims+4},A

	ret


;-------------------------------------------------------
;konwersja na napis w ASCII; poszczegolne kody znakow
;umieszczone w komorkach tablicy DECIMS
;-------------------------------------------------------
dec_2_ascii
	clr	X		;X posluzy jako indeks
	ld	Y,#5		;operacja dla kolejnych 6 bajtow
dec_2_ascii_loop
	ld	A,(decims,X)	;do akumulatora bajt z tablicy
	add	A,#'0'		;dodanie kodu zera
	ld	(decims,X),A	;zapamietanie bajtu w tablicy
	inc	X
	dec	Y		;czy to juz wszystkie bajty?
	jrne	dec_2_ascii_loop

	ret


;-------------------------------------------------------
; program glowny
;-------------------------------------------------------
.main
	bset	TRANCR,#1
	rsp
	call	init
	call	lcd_init
	call	lcd_cls		;czyszczenie LCD, ustawienie kursora 
				;w pozycji HOME
				;wyswietlenie napisow
main0
	call	read_as5040	;odczyt enkodera
	call	bin_2_dec	;konwersja na liczbe dziesietna
	call	dec_2_ascii	;konwersja na ASCII

	ld	A,#$0A		;ustawienie kursora na pozycji 0,0
	call	gotoaxy
	ld	A,#'/';
	call	lcd_data_write

	ld	Y,#5		;Y jako licznik, X jako indeks do pamieci
	clr	X
main2
	ld	A,(decims,X)	;wyswietlenie znakow
	call	lcd_data_write
	inc	X
	dec	Y		;czy wszystkie 5 znakow
	jrne	main2
;	ld	A,#$FF		;pauza ok. 0,2 sekundy
;	ld	del,A
;	call	delay

	jra	main0


;-------------------------------------------------------
; "pusta" funkcja, zawiera tylko instrukcje powrotu
; z obslugi przerwania
;-------------------------------------------------------
.it_ret	iret


	segment 'intvect'
;-------------------------------------------------------
; wektory przerwan
;-------------------------------------------------------
	DC.W	it_ret
	DC.W	it_ret
	DC.W	it_ret
.sci	DC.W	it_ret
.timb	DC.W	timerb
.tima	DC.W	it_ret
.spi	DC.W	it_ret
	DC.W	it_ret
	DC.W	it_ret
.ext3	DC.W	it_ret
.ext2	DC.W	it_ret
.ext1	DC.W	it_ret
.ext0	DC.W	it_ret
	DC.W	it_ret
.soft	DC.W	it_ret
.rst	DC.W	main



	END
